//
// PCM(JR) decoder
//
//                     ---------
//    RS    --> RA2/AN2|1    18| RA1/AN1 -->
//          <-- RA3/AN3|2    17| RA0/AN0 -->
//          <-- RA4/AN4|3    16| OSC1    --
//          /-- RA5    |4    15| OSC2    --
//                  GND|5    14| +5V
//          <-- RB0    |6    13| RB7/AN6 -->
//          <-- RB1    |7    12| RB6/AN5 -->
//          <-- RB2    |8    11| RB5     --> TX
//          <-- RB3    |9    10| RB4     -->
//                     ---------

// for FMS

//#define debug	1


#pragma config  = 0x3F50
// internal 8MHz



//-----------------------------------
#define ERROR			0
#define TRUE			1
#define FALSE			0
#define	HIGH			1
#define LOW				0

#define FALSE                   0
#define TRUE                    1

//-----------------------------------
#pragma bit RA0 @ PORTA.0
#pragma bit RA1 @ PORTA.1
#pragma bit RA2 @ PORTA.2
#pragma bit RA3 @ PORTA.3
#pragma bit RA4 @ PORTA.4
#pragma bit RA5 @ PORTA.5
#pragma bit RA6 @ PORTA.6

#pragma bit RB0 @ PORTB.0
#pragma bit RB1 @ PORTB.1
#pragma bit RB2 @ PORTB.2
#pragma bit RB3 @ PORTB.3
#pragma bit RB4 @ PORTB.4
#pragma bit RB5 @ PORTB.5


//-----------------------------------------------
void	GetPeriod( void );
void	InitUSART( void );

//-----------------------------------------------global
uns16			millisec;
unsigned int	pulse_width;


#include "int16CXX.H"	// device dependent interrupt definitions
//-----------------------------------------------
#pragma origin 4   // start address of interrupt routine (14 bit core)
interrupt serverX( void)
{
    int_save_registers    // W, STATUS (and PCLATH if required)

    //char sv_FSR = FSR;  // save FSR when using tables/pointers
    /* An error is generated if FSR need to be saved and the compiler
       does not detect that this is actually done */

    // handle the interrupt
    if ( TMR1IF )  {
		RB4 = LOW;
		TMR1IE = 0;
        TMR1IF = 0;  /* reset flag */
    }


    //FSR = sv_FSR;               // restore FSR if saved
    int_restore_registers // W, STATUS (and PCLATH if required)
}
// GIE is automatically set to 1 on exit (by RETFIE).


//----------------------------------------------------------------
void GetPeriod(void)
{
	
	if( RA2==LOW ){				// RA2=LOW
		while ( RA2==LOW ){}
	}
	else{						// RA2=HIGH
		while ( RA2==HIGH ){}
	}

	pulse_width = TMR0;		// HIGH pulse width
	TMR0 = 0;

}


//----------------------------------------------------
void InitUSART( void )
{
	BRGH	= 1;		// high speed serial port mode
//	SPBRG	= 51;		// Set 19200 baud for 16MHz EXT oscillator
//	SPBRG	= 16;		// Set 57600 baud for 16MHz EXT oscillator
//	SPBRG	= 8;		// Set 57600 baud for 8MHz oscillator
//	SPBRG	= 51;		// Set 9600 baud for 8MHz oscillator
	SPBRG	= 25;		// Set 19200 baud for 8MHz oscillator
	SYNC	= 0;		// Clear SYNC bit ; Set_Async_Mode;
	SPEN	= 1;		// Set serial port enable
	TX9		= 0;		// 8-bit transmissions
	TXEN	= 1;		// Enable transmission
	RCIE	= 0;		// Rx interrupts disable
	RX9		= 0;		// 8-bit receptions
	CREN	= 0;		// disable reception
}

//----------------------------------------------------
void main(void)
{

	unsigned int	tmp;
	unsigned int	tmp2;
	unsigned long	tmp_long;
	unsigned long	tmp2_long;
	unsigned int	symbol;
	unsigned int	sync;
	unsigned long	bitstream;
	unsigned int	bitcount;
	unsigned int	datacount;
	unsigned int	width;
	unsigned int	data[30];
	unsigned long	aile;
	unsigned long	thro;
	unsigned long	rudd;
	unsigned long	elev;
	unsigned int	aile_8;
	unsigned int	thro_8;
	unsigned int	rudd_8;
	unsigned int	elev_8;
	unsigned int	pulse_high;
	unsigned int	pulse_low;
	unsigned long	aile_servo;

const static signed int jr_symbol[] = {
 -1 , 0, 16, -1, 17, 26, -1, -1, 21, 10,  8, -1, -1, -1, -1, -1,
 23, 14, 12, -1,  4, 27, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 19, 15, 13, -1,  5, 25, -1, -1,  7, 29, 31, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 18, 11,  9, -1,  1, 24, -1, -1,  3, 28, 30, -1, -1, -1, -1, -1,
  2, 20, 22, -1,  6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};
/*
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
 -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};
*/

//------device setting
	OSCCON	= 0x70;		// SCS[1:0]=00 Oscillator mode defined by FOSC[2:0](config)
	PORTA	= 0x00;
	PORTB	= 0x20;		// RB5 = HIGH
	ANSEL	= 0x00;		//AN6,5,4,3,2,1,0 = Digital 
//	ADCON0	= 0xC0;		//
//	ADCON1	= 0x00;		//Left justified, ADCS2=1, reference=Vdd/Vss
	CMCON	= 0x07;		//Comparator setting Not use
// 	OPTION	= 0x85;		//prescaler 1/64
 	OPTION	= 0x84;		//prescaler 1/32
 	TMR0	= 0;
 	TRISA	= 0x04;		// port A  7,6,5,4,3,  1,0=output  2=input
 	TRISB	= 0x00;		// port B  7,6,5,4,3,2,1,0=output
//	ADON	= 1;
//	T1CON = 0x21;	// b5:4 10=1:4prescale  b1 enables timer1 -> 8MHz x 1/4 x 1/4 = 0.5MHz = 2usec
	T1CON = 0x31;	// b5:4 11=1:8prescale  b1 enables timer1 -> 16MHz x 1/4 x 1/8 = 0.5MHz = 2usec

RB0=HIGH;
RB1=HIGH;
RB2=HIGH;
RB3=HIGH;
RB4=LOW;

	sync = 0;
	bitstream = 0;
	bitcount = -1;
	datacount = 0;

//------------
	InitUSART();

#ifndef debug
		// Waiting for 1
		while ( RA2==LOW ){}
		// Waiting for 0
		while ( RA2==HIGH ){}
#endif

	TMR0 = 0;

//------------
	while (1)
	{

#ifndef debug
		GetPeriod();
//TXREG = pulse_width;
#endif


		if( sync == 0 ){
//			tmp = pulse_width * 2 + 5 ;
//			tmp = tmp / 10;
//			if( tmp == 5 ){
			// 25syncpX 400usecɑBulā}1͈̔͂`FbN
			if( pulse_width == 24 || pulse_width == 25 || pulse_width == 26 ){
				sync = 1;
				bitstream = 0;
				bitcount = -1;
				datacount = 0;
RB0=LOW;
			}
		}
		else{	// sync != 0

//TXREG = pulse_width;		// serial out LOW period
			// original  $width = int(($_ *16 / 160) + 0.5);
			// ľܓp
//			tmp = pulse_width + 5;
			tmp = pulse_width + 4;	// 1/166 -> 1/160{␳̖
			//  1/10
			width = tmp / 10;


			tmp_long = bitstream << 1;
			tmp_long = tmp_long + 1;
			tmp2 = width -1;
			bitstream = tmp_long << tmp2;
			bitcount += width;

			if( bitcount >= 8 ){
				bitcount -= 8;
				tmp_long = bitstream >> bitcount;			
				tmp = tmp_long & 0xFF;
				
				if( tmp > 96 ){
					symbol = -1;
				}
				else{
					symbol = jr_symbol[tmp];
				}
				data[datacount++] = symbol;
//				data[datacount++] = jr_symbol[tmp];
//TXREG = symbol;
				
				tmp = datacount - 1;
				if( data[tmp] == 255 ){
					sync = 0;
RB1=LOW;
				}

				// AILE
				// 166 - 515 - 859
				// 1.0ms  1.5ms  2.0ms
				// 760us  1528us 2284us
				// 0x00   0x80   0xFE

				// 166 - 515 - 859
				// x10 /8 /4 -30
				// 21     129    238
				// x10/8 : It may be too sensitive...

				// AILE
				if( datacount == 3 ){
					tmp_long = (uns16)data[1] << 5;
					tmp_long = tmp_long | data[2];
//					tmp2_long = tmp_long * 10;
//					tmp_long = tmp2_long / 32;
//					aile_8 = tmp_long - 30;
					aile_8 = tmp_long / 4;
				};

				// RUDD
				if( datacount == 6 ){
					tmp_long = (uns16)data[4] << 5;
					tmp_long = tmp_long | data[5];
//					tmp2_long = tmp_long * 10;
//					tmp_long = tmp2_long / 32;
//					rudd_8 = tmp_long - 30;
					rudd_8 = tmp_long / 4;
				};

				// THRO
				if( datacount == 18 ){
					tmp_long = (uns16)data[16] << 5;
					tmp_long = tmp_long | data[17];
//					tmp2_long = tmp_long * 10;
//					tmp_long = tmp2_long / 32;
//					thro_8 = tmp_long - 30;
					thro_8 = tmp_long / 4;
				};

				// ELEV
				if( datacount == 21 ){
					tmp_long = (uns16)data[19] << 5;
					tmp_long = tmp_long | data[20];
//					tmp2_long = tmp_long * 10;
//					tmp_long = tmp2_long / 32;
//					elev_8 = tmp_long - 30;
					elev_8 = tmp_long / 4;
				};

				// send 5 bytes serial data
				if( datacount == 22 ){
					TXREG = 0xFF;		// packet start
					while( TXIF==0 ){}			// wait for serial register to be sent
					TXREG = thro_8;		// thro 8bit data
					while( TXIF==0 ){}			// wait for serial register to be sent
					TXREG = aile_8;		// aile 8bit data
					while( TXIF==0 ){}			// wait for serial register to be sent
					TXREG = rudd_8;		// rudd 8bit data
					while( TXIF==0 ){}			// wait for serial register to be sent
					TXREG = elev_8;		// elev 8bit data
					while( TXIF==0 ){}			// wait for serial register to be sent

					sync = 0;
					while ( RA2==LOW ){}
				}

				if( datacount == 30 ){
					sync = 0;
RB3=LOW;
				}

			}	// end of if

		}	// end of else

	}	// end of while

}	// end of main

